@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -26,6 +26,7 @@
 #import "OFArray+Private.h"
 #import "OFConcreteArray.h"
 #import "OFData.h"
+#import "OFJSONRepresentationPrivate.h"
 #import "OFNull.h"
 #import "OFString.h"
 #import "OFSubarray.h"
@@ -34,19 +35,27 @@
 #import "OFInvalidArgumentException.h"
 #import "OFOutOfRangeException.h"
 
-static struct {
-	Class isa;
-} placeholder;
-
-@interface OFArray ()
-- (OFString *)
-    of_JSONRepresentationWithOptions: (OFJSONRepresentationOptions)options
-			       depth: (size_t)depth;
+@interface OFArray () <OFJSONRepresentationPrivate>
 @end
 
 @interface OFPlaceholderArray: OFArray
 @end
 
+@interface OFConcreteArraySingleton: OFConcreteArray
+@end
+
+static struct {
+	Class isa;
+} placeholder;
+
+static OFConcreteArraySingleton *emptyArray;
+
+static void
+emptyArrayInit(void)
+{
+	emptyArray = [[OFConcreteArraySingleton alloc] init];
+}
+
 @implementation OFPlaceholderArray
 #ifdef __clang__
 /* We intentionally don't call into super, so silence the warning. */
@@ -56,7 +65,9 @@ static struct {
 #endif
 - (instancetype)init
 {
-	return (id)[[OFConcreteArray alloc] init];
+	static OFOnceControl onceControl = OFOnceControlInitValue;
+	OFOnce(&onceControl, emptyArrayInit);
+	return (id)emptyArray;
 }
 
 - (instancetype)initWithObject: (id)object
@@ -102,6 +113,10 @@ static struct {
 OF_SINGLETON_METHODS
 @end
 
+@implementation OFConcreteArraySingleton
+OF_SINGLETON_METHODS
+@end
+
 @implementation OFArray
 + (void)initialize
 {
