@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -90,6 +90,7 @@ OF_SINGLETON_METHODS
 {
 	@synchronized (self) {
 		struct OFUTF8StringIvars *ivars;
+		bool containsNull;
 
 		if ([self isMemberOfClass: [OFConstantUTF8String class]])
 			return;
@@ -99,7 +100,7 @@ OF_SINGLETON_METHODS
 		ivars->cStringLength = _cStringLength;
 
 		switch (_OFUTF8StringCheck(ivars->cString, ivars->cStringLength,
-		    &ivars->length, &ivars->containsNull)) {
+		    &ivars->length, &containsNull)) {
 		case 1:
 			ivars->isUTF8 = true;
 			break;
@@ -108,6 +109,8 @@ OF_SINGLETON_METHODS
 			@throw [OFInvalidEncodingException exception];
 		}
 
+		ivars->containsNull = containsNull;
+
 		_cString = (char *)ivars;
 		object_setClass(self, [OFConstantUTF8String class]);
 	}
@@ -245,25 +248,25 @@ OF_SINGLETON_METHODS
 	return [self rangeOfString: string options: options range: range];
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
 {
 	[self finishInitialization];
-	return [self indexOfCharacterFromSet: characterSet];
+	return [self rangeOfCharacterFromSet: characterSet];
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
-			  options: (OFStringSearchOptions)options
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options
 {
 	[self finishInitialization];
-	return [self indexOfCharacterFromSet: characterSet options: options];
+	return [self rangeOfCharacterFromSet: characterSet options: options];
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
-			  options: (OFStringSearchOptions)options
-			    range: (OFRange)range
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options
+			     range: (OFRange)range
 {
 	[self finishInitialization];
-	return [self indexOfCharacterFromSet: characterSet
+	return [self rangeOfCharacterFromSet: characterSet
 				     options: options
 				       range: range];
 }
@@ -432,6 +435,54 @@ OF_SINGLETON_METHODS
 	return self.stringByDeletingLastPathComponent;
 }
 
+- (signed char)charValue
+{
+	[self finishInitialization];
+	return self.charValue;
+}
+
+- (signed char)charValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self charValueWithBase: base];
+}
+
+- (short)shortValue
+{
+	[self finishInitialization];
+	return self.shortValue;
+}
+
+- (short)shortValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self shortValueWithBase: base];
+}
+
+- (int)intValue
+{
+	[self finishInitialization];
+	return self.intValue;
+}
+
+- (int)intValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self intValueWithBase: base];
+}
+
+- (long)longValue
+{
+	[self finishInitialization];
+	return self.longValue;
+}
+
+- (long)longValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self longValueWithBase: base];
+}
+
 - (long long)longLongValue
 {
 	[self finishInitialization];
@@ -444,6 +495,54 @@ OF_SINGLETON_METHODS
 	return [self longLongValueWithBase: base];
 }
 
+- (unsigned char)unsignedCharValue
+{
+	[self finishInitialization];
+	return self.unsignedCharValue;
+}
+
+- (unsigned char)unsignedCharValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self unsignedCharValueWithBase: base];
+}
+
+- (unsigned short)unsignedShortValue
+{
+	[self finishInitialization];
+	return self.unsignedShortValue;
+}
+
+- (unsigned short)unsignedShortValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self unsignedShortValueWithBase: base];
+}
+
+- (unsigned int)unsignedIntValue
+{
+	[self finishInitialization];
+	return self.unsignedIntValue;
+}
+
+- (unsigned int)unsignedIntValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self unsignedIntValueWithBase: base];
+}
+
+- (unsigned long)unsignedLongValue
+{
+	[self finishInitialization];
+	return self.unsignedLongValue;
+}
+
+- (unsigned long)unsignedLongValueWithBase: (unsigned char)base
+{
+	[self finishInitialization];
+	return [self unsignedLongValueWithBase: base];
+}
+
 - (unsigned long long)unsignedLongLongValue
 {
 	[self finishInitialization];
