@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -101,10 +101,10 @@ domainFromHostname(OFString *hostname)
 		ret = nil;
 	} @catch (OFInvalidFormatException *e) {
 		/* Not an IP address -> we can use it if it contains a dot. */
-		size_t pos = [hostname rangeOfString: @"."].location;
+		OFRange range = [hostname rangeOfString: @"."];
 
-		if (pos != OFNotFound)
-			ret = [hostname substringFromIndex: pos + 1];
+		if (range.location != OFNotFound)
+			ret = [hostname substringFromIndex: range.location + 1];
 		else
 			ret = nil;
 	}
@@ -295,12 +295,12 @@ parseNetStackArray(OFString *string)
 	while ((line =
 	    [file readLineWithEncoding: [OFLocale encoding]]) != nil) {
 		OFArray *components, *hosts;
-		size_t pos;
+		OFRange range;
 		OFString *address;
 
-		pos = [line indexOfCharacterFromSet: commentCharacters];
-		if (pos != OFNotFound)
-			line = [line substringToIndex: pos];
+		range = [line rangeOfCharacterFromSet: commentCharacters];
+		if (range.location != OFNotFound)
+			line = [line substringToIndex: range.location];
 
 		components = [line
 		    componentsSeparatedByCharactersInSet: whitespaceCharacterSet
@@ -341,32 +341,17 @@ parseNetStackArray(OFString *string)
 {
 	@try {
 		if ([option hasPrefix: @"ndots:"]) {
-			unsigned long long number;
-
 			option = [option substringFromIndex: 6];
-			number = option.unsignedLongLongValue;
-
-			if (number > UINT_MAX)
-				@throw [OFOutOfRangeException exception];
-
-			_minNumberOfDotsInAbsoluteName = (unsigned int)number;
+			_minNumberOfDotsInAbsoluteName =
+			    option.unsignedIntValue;
 		} else if ([option hasPrefix: @"timeout:"]) {
 			option = [option substringFromIndex: 8];
-
 			_timeout = option.unsignedLongLongValue;
 		} else if ([option hasPrefix: @"attempts:"]) {
-			unsigned long long number;
-
 			option = [option substringFromIndex: 9];
-			number = option.unsignedLongLongValue;
-
-			if (number > UINT_MAX)
-				@throw [OFOutOfRangeException exception];
-
-			_maxAttempts = (unsigned int)number;
+			_maxAttempts = option.unsignedIntValue;
 		} else if ([option hasPrefix: @"reload-period:"]) {
 			option = [option substringFromIndex: 14];
-
 			_configReloadInterval = option.unsignedLongLongValue;
 		} else if ([option isEqual: @"tcp"])
 			_forcesTCP = true;
@@ -398,13 +383,13 @@ parseNetStackArray(OFString *string)
 	while ((line =
 	    [file readLineWithEncoding: [OFLocale encoding]]) != nil) {
 		void *pool2 = objc_autoreleasePoolPush();
-		size_t pos;
+		OFRange range;
 		OFArray *components, *arguments;
 		OFString *option;
 
-		pos = [line indexOfCharacterFromSet: commentCharacters];
-		if (pos != OFNotFound)
-			line = [line substringToIndex: pos];
+		range = [line rangeOfCharacterFromSet: commentCharacters];
+		if (range.location != OFNotFound)
+			line = [line substringToIndex: range.location];
 
 		components = [line
 		    componentsSeparatedByCharactersInSet: whitespaceCharacterSet
