@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -184,13 +184,17 @@
 		sender->length = (socklen_t)sizeof(sender->sockaddr);
 
 #ifndef OF_WINDOWS
-	if ((ret = recvfrom(_socket, buffer, length, 0,
+	while ((ret = recvfrom(_socket, buffer, length, 0,
 	    (sender != NULL ? (struct sockaddr *)&sender->sockaddr : NULL),
-	    (sender != NULL ? &sender->length : NULL))) < 0)
-		@throw [OFReadFailedException
-		    exceptionWithObject: self
-			requestedLength: length
-				  errNo: _OFSocketErrNo()];
+	    (sender != NULL ? &sender->length : NULL))) < 0) {
+		int errNo = _OFSocketErrNo();
+
+		if (errNo != EINTR)
+			@throw [OFReadFailedException
+			    exceptionWithObject: self
+				requestedLength: length
+					  errNo: errNo];
+	}
 #else
 	if (length > INT_MAX)
 		@throw [OFOutOfRangeException exception];
@@ -335,13 +339,17 @@
 	if (length > SSIZE_MAX)
 		@throw [OFOutOfRangeException exception];
 
-	if ((bytesWritten = sendto(_socket, (void *)buffer, length, 0,
-	    (struct sockaddr *)&receiver->sockaddr, receiver->length)) < 0)
-		@throw [OFWriteFailedException
-		    exceptionWithObject: self
-			requestedLength: length
-			   bytesWritten: 0
-				  errNo: _OFSocketErrNo()];
+	while ((bytesWritten = sendto(_socket, (void *)buffer, length, 0,
+	    (struct sockaddr *)&receiver->sockaddr, receiver->length)) < 0) {
+		int errNo = _OFSocketErrNo();
+
+		if (errNo != EINTR)
+			@throw [OFWriteFailedException
+			    exceptionWithObject: self
+				requestedLength: length
+				   bytesWritten: 0
+					  errNo: errNo];
+	}
 #else
 	int bytesWritten;
 
