@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -43,6 +43,7 @@
 # include <sys/extattr.h>
 #endif
 #ifdef OF_HAIKU
+# include <TypeConstants.h>
 # include <kernel/fs_attr.h>
 #endif
 #ifdef OF_DJGPP
@@ -763,6 +764,62 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		fs_close_attr_dir(dir);
 	}
+# elif defined(OF_SOLARIS)
+	int fd;
+	DIR *dir;
+
+	if ((fd = attropen(cPath, ".", O_RDONLY)) == -1)
+		return;
+
+	if ((dir = fdopendir(fd)) == NULL) {
+		close(fd);
+		return;
+	}
+
+#  ifdef OF_HAVE_THREADS
+	@try {
+		[readdirMutex lock];
+	} @catch (id e) {
+		closedir(dir);
+		close(fd);
+		@throw e;
+	}
+#  endif
+
+	@try {
+		names = [OFMutableArray array];
+
+		for (;;) {
+			struct dirent *dirent;
+			OFString *name;
+
+			errno = 0;
+			if ((dirent = readdir(dir)) == NULL) {
+				if (errno == 0)
+					break;
+				else
+					return;
+			}
+
+			if (strcmp(dirent->d_name, ".") == 0 ||
+			    strcmp(dirent->d_name, "..") == 0)
+				continue;
+
+			name = [[OFString alloc] initWithCString: dirent->d_name
+							encoding: encoding];
+			@try {
+				[names addObject: name];
+			} @finally {
+				[name release];
+			}
+		}
+	} @finally {
+#  ifdef OF_HAVE_THREADS
+		[readdirMutex unlock];
+#  endif
+		closedir(dir);
+		close(fd);
+	}
 # endif
 
 	[names makeImmutable];
@@ -977,6 +1034,30 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 			  attributes: attributes
 		     failedAttribute: attributeKey
 			       errNo: lastError()];
+#elif defined(HAVE_UTIMENSAT)
+	OFTimeInterval lastAccessTime = lastAccessDate.timeIntervalSince1970;
+	OFTimeInterval modificationTime =
+	    modificationDate.timeIntervalSince1970;
+	struct timespec times[2] = {
+		{
+			.tv_sec = (time_t)lastAccessTime,
+			.tv_nsec = (int)((lastAccessTime -
+			    (time_t)lastAccessTime) * 1000000000)
+		},
+		{
+			.tv_sec = (time_t)modificationTime,
+			.tv_nsec = (long)((modificationTime -
+			    (time_t)modificationTime) * 1000000000)
+		},
+	};
+
+	if (utimensat(AT_FDCWD, [path cStringWithEncoding: [OFLocale encoding]],
+	    times, AT_SYMLINK_NOFOLLOW) != 0)
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: attributeKey
+			       errNo: errno];
 #else
 	OFTimeInterval lastAccessTime = lastAccessDate.timeIntervalSince1970;
 	OFTimeInterval modificationTime =
@@ -994,7 +1075,12 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		},
 	};
 
+# ifdef HAVE_LUTIMES
+	if (lutimes([path cStringWithEncoding: [OFLocale encoding]], times) !=
+	    0)
+# else
 	if (utimes([path cStringWithEncoding: [OFLocale encoding]], times) != 0)
+# endif
 		@throw [OFSetItemAttributesFailedException
 		    exceptionWithIRI: IRI
 			  attributes: attributes
@@ -1710,9 +1796,9 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	OFString *path = IRI.fileSystemRepresentation;
 	OFStringEncoding encoding = [OFLocale encoding];
 	const char *cPath = [path cStringWithEncoding: encoding];
-	void *value = NULL;
 # if defined(OF_LINUX) || defined(OF_MACOS)
 	const char *cName = [name cStringWithEncoding: encoding];
+	void *value = NULL;
 #  if defined(OF_LINUX)
 	ssize_t size = lgetxattr(cPath, cName, NULL, 0);
 #  elif defined(OF_MACOS)
@@ -1750,6 +1836,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	int namespace;
 	const char *cName;
 	ssize_t size;
+	void *value = NULL;
 
 	parseAttributeName(&name, &namespace);
 	cName = [name cStringWithEncoding: encoding];
@@ -1781,6 +1868,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	const char *cName = [name cStringWithEncoding: encoding];
 	int fd = open(cPath, O_RDONLY);
 	struct attr_info info;
+	void *value = NULL;
 
 	if (fd == -1)
 		@throw [OFGetItemAttributesFailedException
@@ -1816,6 +1904,36 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		OFFreeMemory(value);
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName = [name cStringWithEncoding: encoding];
+	int fd;
+
+	if ((fd = attropen(cPath, cName, O_RDONLY)) == -1)
+		@throw [OFGetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			       errNo: errno];
+
+	@try {
+		OFMutableData *mutableData = [OFMutableData data];
+		char buffer[512];
+		ssize_t length;
+
+		while ((length = read(fd, buffer, 512)) > 0)
+			[mutableData addItems: buffer count: length];
+
+		if (length < 0)
+			@throw [OFGetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				       errNo: errno];
+
+		[mutableData makeImmutable];
+		*data = mutableData;
+	} @finally {
+		close(fd);
+	}
+
+	if (type != NULL)
+		*type = nil;
 # endif
 
 	[*data retain];
@@ -1841,12 +1959,13 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	size_t size = data.count * data.itemSize;
 
 # if defined(OF_LINUX) || defined(OF_MACOS)
-	const char *cName = [name cStringWithEncoding: encoding];
+	const char *cName;
 
 	if (type != nil)
 		@throw [OFNotImplementedException exceptionWithSelector: _cmd
 								 object: self];
 
+	cName = [name cStringWithEncoding: encoding];
 #  if defined(OF_LINUX)
 	if (lsetxattr(cPath, cName, data.items, size, 0) != 0) {
 #  elif defined(OF_MACOS)
@@ -1888,15 +2007,13 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	}
 # elif defined(OF_HAIKU)
 	const char *cName = [name cStringWithEncoding: encoding];
-	unsigned long long typeInt;
+	unsigned int typeInt;
 	int fd;
 
 	if (type != nil && ![type isKindOfClass: [OFNumber class]])
 		@throw [OFInvalidArgumentException exception];
 
-	typeInt = (type != nil ? [type unsignedLongLongValue] : 0);
-	if (typeInt > UINT32_MAX)
-		@throw [OFInvalidArgumentException exception];
+	typeInt = (type != nil ? [type unsignedIntValue] : 0);
 
 	if (size > SSIZE_MAX)
 		@throw [OFOutOfRangeException exception];
@@ -1930,6 +2047,39 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName;
+	int fd;
+
+	if (type != nil)
+		@throw [OFNotImplementedException exceptionWithSelector: _cmd
+								 object: self];
+
+	cName = [name cStringWithEncoding: encoding];
+	fd = attropen(cPath, cName, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+
+	if (fd == -1)
+		/* TODO: Add an attribute (prefix?) for extended attributes? */
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: [OFDictionary dictionary]
+		     failedAttribute: @""
+			       errNo: errno];
+
+	@try {
+		if (write(fd, data.items, size) != (ssize_t)size)
+			/*
+			 * TODO: Add an attribute (prefix?) for extended
+			 *	 attributes?
+			 */
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: [OFDictionary dictionary]
+			     failedAttribute: @""
+				       errNo: errno];
+	} @finally {
+		close(fd);
+	}
 # endif
 
 	objc_autoreleasePoolPop(pool);
@@ -2008,6 +2158,32 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName = [name cStringWithEncoding: encoding];
+	int fd;
+
+	if ((fd = attropen(cPath, ".", O_RDONLY)) < 0)
+		/* TODO: Add an attribute (prefix?) for extended attributes? */
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: [OFDictionary dictionary]
+		     failedAttribute: @""
+			       errNo: errno];
+
+	@try {
+		if (unlinkat(fd, cName, 0) != 0)
+			/*
+			 * TODO: Add an attribute (prefix?) for extended
+			 *	 attributes?
+			 */
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: [OFDictionary dictionary]
+			     failedAttribute: @""
+				       errNo: errno];
+	} @finally {
+		close(fd);
+	}
 # endif
 
 	objc_autoreleasePoolPop(pool);
