@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -97,6 +97,20 @@
 	objc_autoreleasePoolPop(pool);
 }
 
+- (void)setUnsignedLongLong: (unsigned long long)unsignedLongLong
+		    forPath: (OFString *)path
+{
+	void *pool = objc_autoreleasePoolPush();
+	OFString *section, *key;
+
+	[self of_getSection: &section andKey: &key forPath: path];
+	[[_INIFile sectionForName: section]
+	    setUnsignedLongLongValue: unsignedLongLong
+			      forKey: key];
+
+	objc_autoreleasePoolPop(pool);
+}
+
 - (void)setBool: (bool)bool_ forPath: (OFString *)path
 {
 	void *pool = objc_autoreleasePoolPush();
@@ -176,6 +190,23 @@
 	return ret;
 }
 
+- (unsigned long long)unsignedLongLongForPath: (OFString *)path
+				 defaultValue: (unsigned long long)defaultValue
+{
+	void *pool = objc_autoreleasePoolPush();
+	OFString *section, *key;
+	unsigned long long ret;
+
+	[self of_getSection: &section andKey: &key forPath: path];
+	ret = [[_INIFile sectionForName: section]
+	    unsignedLongLongValueForKey: key
+			   defaultValue: defaultValue];
+
+	objc_autoreleasePoolPop(pool);
+
+	return ret;
+}
+
 - (bool)boolForPath: (OFString *)path defaultValue: (bool)defaultValue
 {
 	void *pool = objc_autoreleasePoolPush();
