@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -23,6 +23,7 @@
 #include <string.h>
 
 #import "OFIRI.h"
+#import "OFIRI+Private.h"
 #import "OFArray.h"
 #import "OFDictionary.h"
 #ifdef OF_HAVE_FILES
@@ -37,6 +38,7 @@
 #import "OFInvalidArgumentException.h"
 #import "OFInvalidFormatException.h"
 #import "OFOutOfMemoryException.h"
+#import "OFOutOfRangeException.h"
 
 @interface OFIRIAllowedCharacterSetBase: OFCharacterSet
 @end
@@ -432,7 +434,8 @@ _OFIRIVerifyIsEscaped(OFString *string, OFCharacterSet *characterSet,
 	else
 		characterSet = characterSet.invertedSet;
 
-	if ([string indexOfCharacterFromSet: characterSet] != OFNotFound)
+	if ([string rangeOfCharacterFromSet: characterSet].location !=
+	    OFNotFound)
 		@throw [OFInvalidFormatException exception];
 
 	objc_autoreleasePoolPop(pool);
@@ -559,6 +562,7 @@ static void
 parseHostPort(OFIRI *self, const char *UTF8String, size_t length)
 {
 	OFString *portString;
+	unsigned short port;
 
 	if (*UTF8String == '[') {
 		const char *end = memchr(UTF8String, ']', length);
@@ -615,12 +619,18 @@ parseHostPort(OFIRI *self, const char *UTF8String, size_t length)
 			@throw [OFInvalidFormatException exception];
 
 	portString = [OFString stringWithUTF8String: UTF8String length: length];
-
-	if (portString.unsignedLongLongValue > 65535)
+	@try {
+		port = portString.unsignedShortValue;
+	} @catch (OFOutOfRangeException *e) {
 		@throw [OFInvalidFormatException exception];
+	}
 
-	self->_port = [[OFNumber alloc] initWithUnsignedShort:
-	    (unsigned short)portString.unsignedLongLongValue];
+#if USHRT_MAX != 65535
+	if (port > 65535)
+		@throw [OFInvalidFormatException exception];
+#endif
+
+	self->_port = [[OFNumber alloc] initWithUnsignedShort: port];
 }
 
 static size_t
