@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -84,7 +84,17 @@
 	return [super alloc];
 }
 
++ (bool)handlesForeignEvents
+{
+	return false;
+}
+
 - (instancetype)init
+{
+	return [self initWithRunLoopMode: nil];
+}
+
+- (instancetype)initWithRunLoopMode: (OFRunLoopMode)runLoopMode
 {
 	self = [super init];
 
@@ -198,25 +208,25 @@
 	[_writeObjects removeObjectIdenticalTo: object];
 }
 
-- (bool)of_processReadBuffers
+- (bool)processReadBuffers
 {
 	void *pool = objc_autoreleasePoolPush();
 	bool foundInReadBuffer = false;
 
-	for (id object in [[_readObjects copy] autorelease]) {
+	for (OFStream *stream in [[_readObjects copy] autorelease]) {
 		void *pool2;
 
-		if (![object isKindOfClass: [OFStream class]])
+		if (![stream isKindOfClass: [OFStream class]])
 			continue;
 
 		pool2 = objc_autoreleasePoolPush();
 
-		if ([object hasDataInReadBuffer] &&
-		    (![object of_isWaitingForDelimiter] ||
-		    [object lowlevelHasDataInReadBuffer])) {
+		if (stream.hasDataInReadBuffer &&
+		    (!stream.of_waitingForDelimiter ||
+		    [stream lowlevelHasDataInReadBuffer])) {
 			if ([_delegate respondsToSelector:
 			    @selector(objectIsReadyForReading:)])
-				[_delegate objectIsReadyForReading: object];
+				[_delegate objectIsReadyForReading: stream];
 
 			foundInReadBuffer = true;
 		}
@@ -260,28 +270,16 @@
 
 	Permit();
 #elif defined(OF_HAVE_PIPE)
-	do {
-		if (write(_cancelFD[1], "", 1) == 1)
-			break;
-
+	while (write(_cancelFD[1], "", 1) < 1)
 		OFEnsure(errno == EINTR);
-	} while (true);
 #elif defined(OF_WII)
-	do {
-		if (sendto(_cancelFD[1], "", 1, 0,
-		    (struct sockaddr *)&_cancelAddr, 8) == 1)
-			break;
-
+	while (sendto(_cancelFD[1], "", 1, 0,
+	    (struct sockaddr *)&_cancelAddr, 8) < 1)
 		OFEnsure(_OFSocketErrNo() == EINTR);
-	} while (true);
 #else
-	do {
-		if (sendto(_cancelFD[1], (void *)"", 1, 0,
-		    (struct sockaddr *)&_cancelAddr, sizeof(_cancelAddr)) == 1)
-			break;
-
+	while (sendto(_cancelFD[1], (void *)"", 1, 0,
+	    (struct sockaddr *)&_cancelAddr, sizeof(_cancelAddr)) < 1)
 		OFEnsure(_OFSocketErrNo() == EINTR);
-	} while (true);
 #endif
 }
 @end
