@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -80,6 +80,9 @@ static OFRunLoop *mainRunLoop = nil;
 # endif
 #endif
 }
+
+- (instancetype)init OF_UNAVAILABLE;
+- (instancetype)initWithMode: (OFRunLoopMode)mode;
 @end
 
 #ifdef OF_HAVE_SOCKETS
@@ -241,6 +244,11 @@ static OFRunLoop *mainRunLoop = nil;
 
 @implementation OFRunLoopState
 - (instancetype)init
+{
+	OF_INVALID_INIT_METHOD
+}
+
+- (instancetype)initWithMode: (OFRunLoopMode)mode
 {
 	self = [super init];
 
@@ -253,6 +261,12 @@ static OFRunLoop *mainRunLoop = nil;
 #ifdef OF_HAVE_SOCKETS
 		_readQueues = [[OFMutableDictionary alloc] init];
 		_writeQueues = [[OFMutableDictionary alloc] init];
+
+		if ([OFKernelEventObserver handlesForeignEvents]) {
+			_kernelEventObserver = [[OFKernelEventObserver alloc]
+			    initWithRunLoopMode: mode];
+			_kernelEventObserver.delegate = self;
+		}
 #endif
 #if defined(OF_HAVE_THREADS)
 		_condition = [[OFCondition alloc] init];
@@ -1232,7 +1246,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 		state = [self->_states objectForKey: mode];
 
 		if (create && state == nil) {
-			state = [[OFRunLoopState alloc] init];
+			state = [[OFRunLoopState alloc] initWithMode: mode];
 			@try {
 				[self->_states setObject: state forKey: mode];
 			} @finally {
@@ -1243,7 +1257,8 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 #ifdef OF_HAVE_SOCKETS
 		if (createObserver && state->_kernelEventObserver == nil) {
 			state->_kernelEventObserver =
-			    [[OFKernelEventObserver alloc] init];
+			    [[OFKernelEventObserver alloc]
+			    initWithRunLoopMode: mode];
 			state->_kernelEventObserver.delegate = state;
 		}
 #endif
@@ -1630,7 +1645,8 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 
 		_states = [[OFMutableDictionary alloc] init];
 
-		state = [[OFRunLoopState alloc] init];
+		state = [[OFRunLoopState alloc]
+		    initWithMode: OFDefaultRunLoopMode];
 		@try {
 			[_states setObject: state forKey: OFDefaultRunLoopMode];
 		} @finally {
@@ -1923,14 +1939,8 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 
 #ifdef OF_HAVE_SOCKETS
 			if (state->_kernelEventObserver != nil) {
-				@try {
-					[state->_kernelEventObserver
-					    observeForTimeInterval: timeout];
-				} @catch (OFObserveKernelEventsFailedException
-				    *e) {
-					if (e.errNo != EINTR)
-						@throw e;
-				}
+				[state->_kernelEventObserver
+				    observeForTimeInterval: timeout];
 			} else {
 #endif
 #ifdef OF_HAVE_THREADS
@@ -1961,15 +1971,9 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 			 * another thread, it cancels the observe.
 			 */
 #ifdef OF_HAVE_SOCKETS
-			if (state->_kernelEventObserver != nil) {
-				@try {
-					[state->_kernelEventObserver observe];
-				} @catch (OFObserveKernelEventsFailedException
-				    *e) {
-					if (e.errNo != EINTR)
-						@throw e;
-				}
-			} else {
+			if (state->_kernelEventObserver != nil)
+				[state->_kernelEventObserver observe];
+			else {
 #endif
 #ifdef OF_HAVE_THREADS
 				[state->_condition lock];
