@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -153,11 +153,15 @@
 		@throw [OFNotOpenException exceptionWithObject: self];
 
 #ifndef OF_WINDOWS
-	if ((ret = recv(_socket, buffer, length, 0)) < 0)
-		@throw [OFReadFailedException
-		    exceptionWithObject: self
-			requestedLength: length
-				  errNo: _OFSocketErrNo()];
+	while ((ret = recv(_socket, buffer, length, 0)) < 0) {
+		int errNo = _OFSocketErrNo();
+
+		if (errNo != EINTR)
+			@throw [OFReadFailedException
+			    exceptionWithObject: self
+				requestedLength: length
+					  errNo: errNo];
+	}
 #else
 	if (length > INT_MAX)
 		@throw [OFOutOfRangeException exception];
@@ -268,12 +272,16 @@
 	if (length > SSIZE_MAX)
 		@throw [OFOutOfRangeException exception];
 
-	if ((bytesWritten = send(_socket, (void *)buffer, length, 0)) < 0)
-		@throw [OFWriteFailedException
-		    exceptionWithObject: self
-			requestedLength: length
-			   bytesWritten: 0
-				  errNo: _OFSocketErrNo()];
+	while ((bytesWritten = send(_socket, (void *)buffer, length, 0)) < 0) {
+		int errNo = _OFSocketErrNo();
+
+		if (errNo != EINTR)
+			@throw [OFWriteFailedException
+			    exceptionWithObject: self
+				requestedLength: length
+				   bytesWritten: 0
+					  errNo: errNo];
+	}
 #else
 	int bytesWritten;
 
