@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -397,19 +397,20 @@ OFSocketAddressParseIPv4(OFString *IPv4, uint16_t port)
 	addr = 0;
 
 	for (OFString *component in components) {
-		unsigned long long number;
+		unsigned char number;
 
 		if (component.length == 0)
 			@throw [OFInvalidFormatException exception];
 
-		if ([component indexOfCharacterFromSet:
-		    whitespaceCharacterSet] != OFNotFound)
+		if ([component rangeOfCharacterFromSet:
+		    whitespaceCharacterSet].location != OFNotFound)
 			@throw [OFInvalidFormatException exception];
 
-		number = component.unsignedLongLongValue;
-
-		if (number > UINT8_MAX)
+		@try {
+			number = component.unsignedCharValue;
+		} @catch (OFOutOfRangeException *e) {
 			@throw [OFInvalidFormatException exception];
+		}
 
 		addr = (addr << 8) | ((uint32_t)number & 0xFF);
 	}
@@ -424,18 +425,24 @@ OFSocketAddressParseIPv4(OFString *IPv4, uint16_t port)
 static uint16_t
 parseIPv6Component(OFString *component)
 {
-	unsigned long long number;
+	unsigned short number;
 
-	if ([component indexOfCharacterFromSet:
-	    [OFCharacterSet whitespaceCharacterSet]] != OFNotFound)
+	if ([component rangeOfCharacterFromSet:
+	    [OFCharacterSet whitespaceCharacterSet]].location != OFNotFound)
 		@throw [OFInvalidFormatException exception];
 
-	number = [component unsignedLongLongValueWithBase: 16];
-
-	if (number > UINT16_MAX)
+	@try {
+		number = [component unsignedShortValueWithBase: 16];
+	} @catch (OFOutOfRangeException *e) {
 		@throw [OFInvalidFormatException exception];
+	}
 
-	return (uint16_t)number;
+#if USHRT_MAX != 65535
+	if (number > 65535)
+		@throw [OFInvalidFormatException exception];
+#endif
+
+	return number;
 }
 
 static OFString *
@@ -489,8 +496,8 @@ OFSocketAddressParseIPv6(OFString *IPv6, uint16_t port)
 		IPv6 = [IPv6 substringToIndex: percent];
 
 		@try {
-			addrIn6->sin6_scope_id = (uint32_t)[interface
-			    unsignedLongLongValueWithBase: 10];
+			addrIn6->sin6_scope_id = [interface
+			    unsignedIntValueWithBase: 10];
 		} @catch (OFInvalidFormatException *e) {
 #if defined(HAVE_IF_NAMETOINDEX) && !defined(OF_WINDOWS)
 			addrIn6->sin6_scope_id = if_nametoindex([interface
