@@ -46,6 +46,7 @@
 #endif
 #import "OFIRI.h"
 #import "OFIRIHandler.h"
+#import "OFJSONRepresentationPrivate.h"
 #import "OFLocale.h"
 #import "OFStream.h"
 #import "OFSystemInfo.h"
@@ -99,18 +100,16 @@ static locale_t cLocale;
 # endif
 #endif
 
-@interface OFString ()
+OF_DIRECT_MEMBERS
+@interface OFString () <OFJSONRepresentationPrivate>
 - (size_t)of_getCString: (char *)cString
 	      maxLength: (size_t)maxLength
 	       encoding: (OFStringEncoding)encoding
 		  lossy: (bool)lossy
-	       insecure: (bool)insecure OF_DIRECT;
+	       insecure: (bool)insecure;
 - (const char *)of_cStringWithEncoding: (OFStringEncoding)encoding
 				 lossy: (bool)lossy
-			      insecure: (bool)insecure OF_DIRECT;
-- (OFString *)
-    of_JSONRepresentationWithOptions: (OFJSONRepresentationOptions)options
-			       depth: (size_t)depth;
+			      insecure: (bool)insecure;
 @end
 
 @interface OFPlaceholderString: OFString
@@ -1986,24 +1985,24 @@ OF_SINGLETON_METHODS
 	return OFMakeRange(OFNotFound, 0);
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
 {
-	return [self indexOfCharacterFromSet: characterSet
+	return [self rangeOfCharacterFromSet: characterSet
 				     options: 0
 				       range: OFMakeRange(0, self.length)];
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
-			  options: (OFStringSearchOptions)options
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options
 {
-	return [self indexOfCharacterFromSet: characterSet
+	return [self rangeOfCharacterFromSet: characterSet
 				     options: options
 				       range: OFMakeRange(0, self.length)];
 }
 
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
-			  options: (OFStringSearchOptions)options
-			    range: (OFRange)range
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options
+			     range: (OFRange)range
 {
 	bool (*characterIsMember)(id, SEL, OFUnichar) =
 	    (bool (*)(id, SEL, OFUnichar))[characterSet
@@ -2011,7 +2010,7 @@ OF_SINGLETON_METHODS
 	OFUnichar *characters;
 
 	if (range.length == 0)
-		return OFNotFound;
+		return OFMakeRange(OFNotFound, 0);
 
 	if (range.length > SIZE_MAX / sizeof(OFUnichar))
 		@throw [OFOutOfRangeException exception];
@@ -2025,7 +2024,8 @@ OF_SINGLETON_METHODS
 				if (characterIsMember(characterSet,
 				    @selector(characterIsMember:),
 				    characters[i]))
-					return range.location + i;
+					return OFMakeRange(
+					    range.location + i, 1);
 
 				/* No match and we're at the last character */
 				if (i == 0)
@@ -2036,13 +2036,40 @@ OF_SINGLETON_METHODS
 				if (characterIsMember(characterSet,
 				    @selector(characterIsMember:),
 				    characters[i]))
-					return range.location + i;
+					return OFMakeRange(
+					    range.location + i, 1);
 		}
 	} @finally {
 		OFFreeMemory(characters);
 	}
 
-	return OFNotFound;
+	return OFMakeRange(OFNotFound, 0);
+}
+
+- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+{
+	return [self
+	    rangeOfCharacterFromSet: characterSet
+			    options: 0
+			      range: OFMakeRange(0, self.length)].location;
+}
+
+- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+			  options: (OFStringSearchOptions)options
+{
+	return [self
+	    rangeOfCharacterFromSet: characterSet
+			    options: options
+			      range: OFMakeRange(0, self.length)].location;
+}
+
+- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+			  options: (OFStringSearchOptions)options
+			    range: (OFRange)range
+{
+	return [self rangeOfCharacterFromSet: characterSet
+				     options: options
+				       range: range].location;
 }
 
 - (bool)containsString: (OFString *)string
@@ -2375,91 +2402,13 @@ OF_SINGLETON_METHODS
 	return array;
 }
 
-- (long long)longLongValue
-{
-	return [self longLongValueWithBase: 10];
-}
-
-- (long long)longLongValueWithBase: (unsigned char)base
+static long long
+longLongValueWithBase(OFString *self, unsigned char base, long long min,
+    long long max)
 {
 	void *pool = objc_autoreleasePoolPush();
 	const char *UTF8String = self.UTF8String;
 	bool negative = false;
-	long long value = 0;
-
-	while (OFASCIIIsSpace(*UTF8String))
-		UTF8String++;
-
-	switch (*UTF8String) {
-	case '-':
-		negative = true;
-	case '+':
-		UTF8String++;
-	}
-
-	if (UTF8String[0] == '0') {
-		if (UTF8String[1] == 'x') {
-			if (base == 0)
-				base = 16;
-
-			if (base != 16 || UTF8String[2] == '\0')
-				@throw [OFInvalidFormatException exception];
-
-			UTF8String += 2;
-		} else {
-			if (base == 0)
-				base = 8;
-
-			UTF8String++;
-		}
-	}
-
-	if (base == 0)
-		base = 10;
-
-	while (*UTF8String != '\0') {
-		unsigned char c = OFASCIIToUpper(*UTF8String++);
-
-		if (c >= '0' && c <= '9')
-			c -= '0';
-		else if (c >= 'A' && c <= 'Z')
-			c -= ('A' - 10);
-		else if (OFASCIIIsSpace(c)) {
-			while (*UTF8String != '\0')
-				if (!OFASCIIIsSpace(*UTF8String++))
-					@throw [OFInvalidFormatException
-					    exception];
-
-			break;
-		} else
-			@throw [OFInvalidFormatException exception];
-
-		if (c >= base)
-			@throw [OFInvalidFormatException exception];
-
-		if (LLONG_MAX / base < value || LLONG_MAX - (value * base) < c)
-			@throw [OFOutOfRangeException exception];
-
-		value = (value * base) + c;
-	}
-
-	if (negative)
-		value *= -1;
-
-	objc_autoreleasePoolPop(pool);
-
-	return value;
-}
-
-- (unsigned long long)unsignedLongLongValue
-{
-	return [self unsignedLongLongValueWithBase: 10];
-}
-
-- (unsigned long long)unsignedLongLongValueWithBase: (unsigned char)base
-{
-	void *pool = objc_autoreleasePoolPush();
-	const char *UTF8String = self.UTF8String;
 	unsigned long long value = 0;
 
 	while (OFASCIIIsSpace(*UTF8String))
@@ -2467,7 +2416,7 @@ OF_SINGLETON_METHODS
 
 	switch (*UTF8String) {
 	case '-':
-		@throw [OFInvalidFormatException exception];
+		negative = true;
 	case '+':
 		UTF8String++;
 	}
@@ -2521,9 +2470,197 @@ OF_SINGLETON_METHODS
 
 	objc_autoreleasePoolPop(pool);
 
+	if (negative) {
+		if (value > -(unsigned long long)min)
+			@throw [OFOutOfRangeException exception];
+
+		return (long long)-value;
+	} else {
+		if (value > (unsigned long long)max)
+			@throw [OFOutOfRangeException exception];
+
+		return (long long)value;
+	}
+}
+
+- (signed char)charValue
+{
+	return (signed char)longLongValueWithBase(
+	    self, 10, SCHAR_MIN, SCHAR_MAX);
+}
+
+- (signed char)charValueWithBase: (unsigned char)base
+{
+	return (signed char)longLongValueWithBase(
+	    self, base, SCHAR_MIN, SCHAR_MAX);
+}
+
+- (short)shortValue
+{
+	return (short)longLongValueWithBase(self, 10, SHRT_MIN, SHRT_MAX);
+}
+
+- (short)shortValueWithBase: (unsigned char)base
+{
+	return (short)longLongValueWithBase(self, base, SHRT_MIN, SHRT_MAX);
+}
+
+- (int)intValue
+{
+	return (int)longLongValueWithBase(self, 10, INT_MIN, INT_MAX);
+}
+
+- (int)intValueWithBase: (unsigned char)base
+{
+	return (int)longLongValueWithBase(self, base, INT_MIN, INT_MAX);
+}
+
+- (long)longValue
+{
+	return (long)longLongValueWithBase(self, 10, LONG_MIN, LONG_MAX);
+}
+
+- (long)longValueWithBase: (unsigned char)base
+{
+	return (long)longLongValueWithBase(self, base, LONG_MIN, LONG_MAX);
+}
+
+- (long long)longLongValue
+{
+	return longLongValueWithBase(self, 10, LLONG_MIN, LLONG_MAX);
+}
+
+- (long long)longLongValueWithBase: (unsigned char)base
+{
+	return longLongValueWithBase(self, base, LLONG_MIN, LLONG_MAX);
+}
+
+static unsigned long long
+unsignedLongLongValueWithBase(OFString *self, unsigned char base,
+    unsigned long long max)
+{
+	void *pool = objc_autoreleasePoolPush();
+	const char *UTF8String = self.UTF8String;
+	unsigned long long value = 0;
+
+	while (OFASCIIIsSpace(*UTF8String))
+		UTF8String++;
+
+	switch (*UTF8String) {
+	case '-':
+		@throw [OFOutOfRangeException exception];
+	case '+':
+		UTF8String++;
+	}
+
+	if (UTF8String[0] == '0') {
+		if (UTF8String[1] == 'x') {
+			if (base == 0)
+				base = 16;
+
+			if (base != 16 || UTF8String[2] == '\0')
+				@throw [OFInvalidFormatException exception];
+
+			UTF8String += 2;
+		} else {
+			if (base == 0)
+				base = 8;
+
+			UTF8String++;
+		}
+	}
+
+	if (base == 0)
+		base = 10;
+
+	while (*UTF8String != '\0') {
+		unsigned char c = OFASCIIToUpper(*UTF8String++);
+
+		if (c >= '0' && c <= '9')
+			c -= '0';
+		else if (c >= 'A' && c <= 'Z')
+			c -= ('A' - 10);
+		else if (OFASCIIIsSpace(c)) {
+			while (*UTF8String != '\0')
+				if (!OFASCIIIsSpace(*UTF8String++))
+					@throw [OFInvalidFormatException
+					    exception];
+
+			break;
+		} else
+			@throw [OFInvalidFormatException exception];
+
+		if (c >= base)
+			@throw [OFInvalidFormatException exception];
+
+		if (max / base < value || max - (value * base) < c)
+			@throw [OFOutOfRangeException exception];
+
+		value = (value * base) + c;
+	}
+
+	objc_autoreleasePoolPop(pool);
+
 	return value;
 }
 
+- (unsigned char)unsignedCharValue
+{
+	return (unsigned char)unsignedLongLongValueWithBase(
+	    self, 10, UCHAR_MAX);
+}
+
+- (unsigned char)unsignedCharValueWithBase: (unsigned char)base
+{
+	return (unsigned char)unsignedLongLongValueWithBase(
+	    self, base, UCHAR_MAX);
+}
+
+- (unsigned short)unsignedShortValue
+{
+	return (unsigned short)unsignedLongLongValueWithBase(
+	    self, 10, USHRT_MAX);
+}
+
+- (unsigned short)unsignedShortValueWithBase: (unsigned char)base
+{
+	return (unsigned short)unsignedLongLongValueWithBase(
+	    self, base, USHRT_MAX);
+}
+
+- (unsigned int)unsignedIntValue
+{
+	return (unsigned int)unsignedLongLongValueWithBase(self, 10, UINT_MAX);
+}
+
+- (unsigned int)unsignedIntValueWithBase: (unsigned char)base
+{
+	return (unsigned int)unsignedLongLongValueWithBase(
+	    self, base, UINT_MAX);
+}
+
+- (unsigned long)unsignedLongValue
+{
+	return (unsigned long)unsignedLongLongValueWithBase(
+	    self, 10, ULONG_MAX);
+}
+
+- (unsigned long)unsignedLongValueWithBase: (unsigned char)base
+{
+	return (unsigned long)unsignedLongLongValueWithBase(
+	    self, base, ULONG_MAX);
+}
+
+- (unsigned long long)unsignedLongLongValue
+{
+	return unsignedLongLongValueWithBase(self, 10, ULLONG_MAX);
+}
+
+- (unsigned long long)unsignedLongLongValueWithBase: (unsigned char)base
+{
+	return unsignedLongLongValueWithBase(self, base, ULLONG_MAX);
+}
+
 - (float)floatValue
 {
 	void *pool = objc_autoreleasePoolPush();
