@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -560,7 +560,7 @@ cpucfg(uint32_t word)
 			  isDirectory: true];
 # elif defined(OF_AMIGAOS)
 	return [OFIRI fileIRIWithPath: @"PROGDIR:" isDirectory: true];
-# elif defined(OF_WII) || defined(OF_NINTENDO_3DS)
+# elif defined(OF_WII) || defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS)
 	return [[OFFileManager defaultManager] currentDirectoryIRI];
 # else
 	OFDictionary *env = [OFApplication environment];
@@ -652,7 +652,7 @@ cpucfg(uint32_t word)
 			  isDirectory: true];
 # elif defined(OF_AMIGAOS)
 	return [OFIRI fileIRIWithPath: @"PROGDIR:" isDirectory: true];
-# elif defined(OF_WII) || defined(OF_NINTENDO_3DS)
+# elif defined(OF_WII) || defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS)
 	return [[OFFileManager defaultManager] currentDirectoryIRI];
 # else
 	OFDictionary *env = [OFApplication environment];
@@ -731,7 +731,7 @@ cpucfg(uint32_t word)
 	return [OFIRI fileIRIWithPath: path isDirectory: true];
 # elif defined(OF_MINT)
 	return [OFIRI fileIRIWithPath: @"u:\\tmp" isDirectory: true];
-# elif defined(OF_WII) || defined(OF_NINTENDO_3DS)
+# elif defined(OF_WII) || defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS)
 	return [[OFFileManager defaultManager] currentDirectoryIRI];
 # elif defined(OF_NINTENDO_SWITCH)
 	static OFOnceControl onceControl = OFOnceControlInitValue;
