@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -33,11 +33,10 @@ OF_ASSUME_NONNULL_BEGIN
 	struct OFUTF8StringIvars {
 		char          *cString;
 		size_t        cStringLength;
-		bool          isUTF8, containsNull;
 		size_t        length;
-		bool          hasHash;
 		unsigned long hash;
-		bool          freeWhenDone;
+		unsigned      isUTF8: 1, containsNull: 1, hasHash: 1;
+		unsigned      freeWhenDone: 1;
 	} *restrict _s;
 	struct OFUTF8StringIvars _storage;
 }
