@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -87,6 +87,7 @@
 # import "OFTCPSocket.h"
 # import "OFUDPSocket.h"
 # import "OFTLSStream.h"
+# import "OFX509Certificate.h"
 # import "OFKernelEventObserver.h"
 # import "OFDNSQuery.h"
 # import "OFDNSResourceRecord.h"
@@ -181,7 +182,8 @@
 #import "OFInvalidJSONException.h"
 #import "OFInvalidServerResponseException.h"
 #import "OFLinkItemFailedException.h"
-#ifdef OF_HAVE_PLUGINS
+#ifdef OF_HAVE_MODULES
+# import "OFLoadModuleFailedException.h"
 # import "OFLoadPluginFailedException.h"
 #endif
 #import "OFLockFailedException.h"
@@ -241,7 +243,8 @@
 # import "OFThreadStillRunningException.h"
 # import "OFWaitForConditionFailedException.h"
 #endif
-#ifdef OF_HAVE_PLUGINS
+#ifdef OF_HAVE_MODULES
+# import "OFModule.h"
 # import "OFPlugin.h"
 #endif
 #ifdef OF_WINDOWS
