@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import <Foundation/NSArray.h>
 
-#import "NSBridging.h"
+#import "OFNSToOFBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -34,12 +34,11 @@ extern int _NSArray_OFObject_reference OF_VISIBILITY_HIDDEN;
 #endif
 
 /**
- * @category NSArray (OFObject) \
- *	     NSArray+OFObject.h ObjFWBridge/NSArray+OFObject.h
+ * @category NSArray (OFObject) NSArray+OFObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging NSArrays to OFArrays.
+ * @brief Support for bridging an NSArray to an @ref OFArray.
  */
-@interface NSArray (OFObject) <NSBridging>
+@interface NSArray (OFObject) <OFNSToOFBridging>
 @property (readonly, nonatomic) OFArray *OFObject;
 @end
 
