@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import <Foundation/NSValue.h>
 
-#import "NSBridging.h"
+#import "OFNSToOFBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -35,11 +35,11 @@ extern int _NSNumber_OFObject_reference OF_VISIBILITY_HIDDEN;
 
 /**
  * @category NSNumber (OFObject)
- *	     NSNumber+OFObject.h ObjFWBridge/NSNumber+OFObject.h
+ *	     NSNumber+OFObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging NSNumbers to OFNumbers.
+ * @brief Support for bridging an NSNumber to an @ref OFNumber.
  */
-@interface NSNumber (OFObject) <NSBridging>
+@interface NSNumber (OFObject) <OFNSToOFBridging>
 @property (readonly, nonatomic) OFNumber *OFObject;
 @end
 
