@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import "NSOFArray.h"
 #import "OFArray.h"
-#import "OFBridging.h"
+#import "OFOFToNSBridging.h"
 
 #import "OFOutOfRangeException.h"
 
@@ -43,7 +43,8 @@
 {
 	id object = [_array objectAtIndex: idx];
 
-	if ([(OFObject *)object conformsToProtocol: @protocol(OFBridging)])
+	if ([(id <OFObject>)object conformsToProtocol:
+	    @protocol(OFOFToNSBridging)])
 		return [object NSObject];
 
 	return object;
