@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import <Foundation/NSString.h>
 
-#import "NSBridging.h"
+#import "OFNSToOFBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -34,16 +34,15 @@ extern int _NSString_OFObject_reference OF_VISIBILITY_HIDDEN;
 #endif
 
 /**
- * @category NSString (OFObject)
- *	     NSString+OFObject.h ObjFWBridge/NSString+OFObject.h
+ * @category NSString (OFObject) NSString+OFObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging NSStrings to OFStrings.
+ * @brief Support for bridging an NSString to an @ref OFString.
  *
  * Unfortunately, they need to be copied, as NSString is not capable of
  * handling UCS-4 properly (a character of NSString is only 2 bytes, while a
  * character of OFString is 4).
  */
-@interface NSString (OFObject) <NSBridging>
+@interface NSString (OFObject) <OFNSToOFBridging>
 @property (readonly, nonatomic) OFString *OFObject;
 @end
 
