@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,7 +27,7 @@
 # endif
 #endif
 
-#import "OFBridging.h"
+#import "OFOFToNSBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -40,12 +40,12 @@ extern int _OFDictionary_NSObject_reference OF_VISIBILITY_HIDDEN;
 #endif
 
 /**
- * @category OFDictionary (NSObject) \
- *	     OFDictionary+NSObject.h ObjFWBridge/OFDictionary+NSObject.h
+ * @category OFDictionary (NSObject)
+ *	     OFDictionary+NSObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging OFDictionaries to NSDictionaries.
+ * @brief Support for bridging an @ref OFDictionary to an NSDictionary.
  */
-@interface OFDictionary (NSObject) <OFBridging>
+@interface OFDictionary (NSObject) <OFOFToNSBridging>
 @property (readonly, nonatomic) NSDictionary *NSObject;
 @end
 
