@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,7 +27,7 @@
 # endif
 #endif
 
-#import "OFBridging.h"
+#import "OFOFToNSBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -40,11 +40,12 @@ extern int _OFEnumerator_NSObject_reference OF_VISIBILITY_HIDDEN;
 #endif
 
 /**
- * @category OFEnumerator (NSObject) \
- *	     OFEnumerator+NSObject.h ObjFWBridge/OFEnumerator+NSObject.h
- * @brief Support for bridging OFEnumerators to NSEnumerators.
+ * @category OFEnumerator (NSObject)
+ *	     OFEnumerator+NSObject.h ObjFWBridge/ObjFWBridge.h
+ *
+ * @brief Support for bridging an @ref OFEnumerator to an NSEnumerator.
  */
-@interface OFEnumerator (NSObject) <OFBridging>
+@interface OFEnumerator (NSObject) <OFOFToNSBridging>
 @property (readonly, nonatomic) NSEnumerator *NSObject;
 @end
 
