@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,7 +27,7 @@
 # endif
 #endif
 
-#import "OFBridging.h"
+#import "OFOFToNSBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -40,11 +40,11 @@ extern int _OFSet_NSObject_reference OF_VISIBILITY_HIDDEN;
 #endif
 
 /**
- * @category OFSet (NSObject) \
- *	     OFSet+NSObject.h ObjFWBridge/OFSet+NSObject.h
- * @brief Support for bridging OFSets to NSSets.
+ * @category OFSet (NSObject) OFSet+NSObject.h ObjFWBridge/ObjFWBridge.h
+ *
+ * @brief Support for bridging an @ref OFSet to an NSSet.
  */
-@interface OFSet (NSObject) <OFBridging>
+@interface OFSet (NSObject) <OFOFToNSBridging>
 @property (readonly, nonatomic) NSSet *NSObject;
 @end
 
