@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -479,30 +479,45 @@ codepageToEncoding(UINT codepage)
 	return -1;
 }
 
+- (int)colors
+{
+	return 16;
+}
+
 - (void)setForegroundColor: (OFColor *)color
 {
 	CONSOLE_SCREEN_BUFFER_INFO csbi;
 	float red, green, blue;
 
+	if (color == _foregroundColor)
+		return;
+
 	if (!GetConsoleScreenBufferInfo(_handle, &csbi))
 		return;
 
 	csbi.wAttributes &= ~(FOREGROUND_RED | FOREGROUND_GREEN |
 	    FOREGROUND_BLUE | FOREGROUND_INTENSITY);
 
-	[color getRed: &red green: &green blue: &blue alpha: NULL];
+	if (color != nil) {
+		[color getRed: &red green: &green blue: &blue alpha: NULL];
 
-	if (red >= 0.25)
-		csbi.wAttributes |= FOREGROUND_RED;
-	if (green >= 0.25)
-		csbi.wAttributes |= FOREGROUND_GREEN;
-	if (blue >= 0.25)
-		csbi.wAttributes |= FOREGROUND_BLUE;
+		if (red >= 0.25)
+			csbi.wAttributes |= FOREGROUND_RED;
+		if (green >= 0.25)
+			csbi.wAttributes |= FOREGROUND_GREEN;
+		if (blue >= 0.25)
+			csbi.wAttributes |= FOREGROUND_BLUE;
 
-	if (red >= 0.75 || green >= 0.75 || blue >= 0.75)
-		csbi.wAttributes |= FOREGROUND_INTENSITY;
+		if (red >= 0.75 || green >= 0.75 || blue >= 0.75)
+			csbi.wAttributes |= FOREGROUND_INTENSITY;
+	} else
+		csbi.wAttributes |= _attributes & (FOREGROUND_RED |
+		    FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY);
 
-	SetConsoleTextAttribute(_handle, csbi.wAttributes);
+	if (SetConsoleTextAttribute(_handle, csbi.wAttributes)) {
+		[_foregroundColor release];
+		_foregroundColor = [color retain];
+	}
 }
 
 - (void)setBackgroundColor: (OFColor *)color
@@ -510,30 +525,64 @@ codepageToEncoding(UINT codepage)
 	CONSOLE_SCREEN_BUFFER_INFO csbi;
 	float red, green, blue;
 
+	if (color == _backgroundColor)
+		return;
+
 	if (!GetConsoleScreenBufferInfo(_handle, &csbi))
 		return;
 
 	csbi.wAttributes &= ~(BACKGROUND_RED | BACKGROUND_GREEN |
 	    BACKGROUND_BLUE | BACKGROUND_INTENSITY);
 
-	[color getRed: &red green: &green blue: &blue alpha: NULL];
+	if (color != nil) {
+		[color getRed: &red green: &green blue: &blue alpha: NULL];
 
-	if (red >= 0.25)
-		csbi.wAttributes |= BACKGROUND_RED;
-	if (green >= 0.25)
-		csbi.wAttributes |= BACKGROUND_GREEN;
-	if (blue >= 0.25)
-		csbi.wAttributes |= BACKGROUND_BLUE;
+		if (red >= 0.25)
+			csbi.wAttributes |= BACKGROUND_RED;
+		if (green >= 0.25)
+			csbi.wAttributes |= BACKGROUND_GREEN;
+		if (blue >= 0.25)
+			csbi.wAttributes |= BACKGROUND_BLUE;
 
-	if (red >= 0.75 || green >= 0.75 || blue >= 0.75)
-		csbi.wAttributes |= BACKGROUND_INTENSITY;
+		if (red >= 0.75 || green >= 0.75 || blue >= 0.75)
+			csbi.wAttributes |= BACKGROUND_INTENSITY;
+	} else
+		csbi.wAttributes |= _attributes & (BACKGROUND_RED |
+		    BACKGROUND_GREEN | BACKGROUND_BLUE | BACKGROUND_INTENSITY);
 
-	SetConsoleTextAttribute(_handle, csbi.wAttributes);
+	if (SetConsoleTextAttribute(_handle, csbi.wAttributes)) {
+		[_backgroundColor release];
+		_backgroundColor = [color retain];
+	}
+}
+
+- (void)setBold: (bool)bold
+{
+	/* No support for bold. */
+}
+
+- (void)setItalic: (bool)italic
+{
+	/* No support for italic. */
+}
+
+- (void)setUnderlined: (bool)underlined
+{
+	/* No support for underlined. */
+}
+
+- (void)setBlinking: (bool)blinking
+{
+	/* No support for blinking. */
 }
 
 - (void)reset
 {
-	SetConsoleTextAttribute(_handle, _attributes);
+	if (SetConsoleTextAttribute(_handle, _attributes)) {
+		[_foregroundColor release];
+		[_backgroundColor release];
+		_foregroundColor = _backgroundColor = nil;
+	}
 }
 
 - (void)clear
