@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -22,7 +22,11 @@
 #include <errno.h>
 
 #import "OFOpenSSLTLSStream.h"
+#import "OFArray.h"
 #import "OFData.h"
+#import "OFOpenSSLX509Certificate.h"
+
+#include <openssl/err.h>
 
 #import "OFAlreadyOpenException.h"
 #import "OFInitializationFailedException.h"
@@ -34,7 +38,45 @@
 #define bufferSize OFOpenSSLTLSStreamBufferSize
 
 int _ObjFWTLS_reference;
-static SSL_CTX *clientContext;
+static SSL_CTX *clientContext, *serverContext;
+
+static OFTLSStreamErrorCode
+verifyResultToErrorCode(const SSL *SSL_)
+{
+	switch (SSL_get_verify_result(SSL_)) {
+	case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
+	case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
+	case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
+	case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
+	case X509_V_ERR_CERT_UNTRUSTED:
+		return OFTLSStreamErrorCodeCertificateIssuerUntrusted;
+	case X509_V_ERR_HOSTNAME_MISMATCH:
+		return OFTLSStreamErrorCodeCertificateNameMismatch;
+	case X509_V_ERR_CERT_NOT_YET_VALID:
+	case X509_V_ERR_CERT_HAS_EXPIRED:
+		return OFTLSStreamErrorCodeCertificatedExpired;
+	case X509_V_ERR_CERT_REVOKED:
+		return OFTLSStreamErrorCodeCertificateRevoked;
+	}
+
+	return OFTLSStreamErrorCodeCertificateVerificationFailed;
+}
+
+static OFTLSStreamErrorCode
+errToErrorCode(const SSL *SSL_)
+{
+	unsigned long err = ERR_get_error();
+
+	switch (ERR_GET_LIB(err)) {
+	case ERR_LIB_SSL:
+		switch (ERR_GET_REASON(err)) {
+		case SSL_R_CERTIFICATE_VERIFY_FAILED:
+			return verifyResultToErrorCode(SSL_);
+		}
+	}
+
+	return OFTLSStreamErrorCodeUnknown;
+}
 
 @implementation OFOpenSSLTLSStream
 + (void)load
@@ -55,6 +97,10 @@ static SSL_CTX *clientContext;
 	    SSL_CTX_set_default_verify_paths(clientContext) != 1)
 		@throw [OFInitializationFailedException
 		    exceptionWithClass: self];
+
+	if ((serverContext = SSL_CTX_new(TLS_server_method())) == NULL)
+		@throw [OFInitializationFailedException
+		    exceptionWithClass: self];
 }
 
 - (instancetype)initWithStream: (OFStream <OFReadyForReadingObserving,
@@ -124,6 +170,7 @@ static SSL_CTX *clientContext;
 	if (!_handshakeDone)
 		@throw [OFNotOpenException exceptionWithObject: self];
 
+	ERR_clear_error();
 	ret = SSL_read_ex(_SSL, buffer, length, &bytesRead);
 
 	while (BIO_ctrl_pending(_writeBIO) > 0) {
@@ -155,6 +202,7 @@ static SSL_CTX *clientContext;
 			}
 		}
 
+		ERR_clear_error();
 		ret = SSL_read_ex(_SSL, buffer, length, &bytesRead);
 
 		while (BIO_ctrl_pending(_writeBIO) > 0) {
@@ -187,6 +235,8 @@ static SSL_CTX *clientContext;
 	if (!_handshakeDone)
 		@throw [OFNotOpenException exceptionWithObject: self];
 
+	ERR_clear_error();
+
 	if ((ret = SSL_write_ex(_SSL, buffer, length, &bytesWritten)) != 1) {
 		/* FIXME: Translate error to errNo */
 		int errNo = 0;
@@ -223,8 +273,9 @@ static SSL_CTX *clientContext;
 #endif
 }
 
-- (void)asyncPerformClientHandshakeWithHost: (OFString *)host
-				runLoopMode: (OFRunLoopMode)runLoopMode
+- (void)of_asyncPerformHandshakeWithHost: (OFString *)host
+				  server: (bool)server
+			     runLoopMode: (OFRunLoopMode)runLoopMode
 {
 	static const OFTLSStreamErrorCode initFailedErrorCode =
 	    OFTLSStreamErrorCodeInitializationFailed;
@@ -252,7 +303,7 @@ static SSL_CTX *clientContext;
 	BIO_set_mem_eof_return(_readBIO, -1);
 	BIO_set_mem_eof_return(_writeBIO, -1);
 
-	if ((_SSL = SSL_new(clientContext)) == NULL) {
+	if ((_SSL = SSL_new(server ? serverContext : clientContext)) == NULL) {
 		BIO_free(_readBIO);
 		BIO_free(_writeBIO);
 		@throw [OFTLSHandshakeFailedException
@@ -262,26 +313,61 @@ static SSL_CTX *clientContext;
 	}
 
 	SSL_set_bio(_SSL, _readBIO, _writeBIO);
-	SSL_set_connect_state(_SSL);
+
+	if (server)
+		SSL_set_accept_state(_SSL);
+	else
+		SSL_set_connect_state(_SSL);
 
 	_host = [host copy];
+	_server = server;
 
-	if (SSL_set_tlsext_host_name(_SSL, _host.UTF8String) != 1)
-		@throw [OFTLSHandshakeFailedException
-		    exceptionWithStream: self
-				   host: host
-			      errorCode: initFailedErrorCode];
-
-	if (_verifiesCertificates) {
-		SSL_set_verify(_SSL, SSL_VERIFY_PEER, NULL);
-
-		if (SSL_set1_host(_SSL, _host.UTF8String) != 1)
+	if (!server) {
+		if (SSL_set_tlsext_host_name(_SSL, _host.UTF8String) != 1)
 			@throw [OFTLSHandshakeFailedException
 			    exceptionWithStream: self
 					   host: host
 				      errorCode: initFailedErrorCode];
+
+		if (_verifiesCertificates) {
+			SSL_set_verify(_SSL, SSL_VERIFY_PEER, NULL);
+
+			if (SSL_set1_host(_SSL, _host.UTF8String) != 1)
+				@throw [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: host
+					      errorCode: initFailedErrorCode];
+		}
 	}
 
+	if (_certificateChain.count > 0) {
+		OFOpenSSLX509Certificate *certificate =
+		    (OFOpenSSLX509Certificate *)_certificateChain.firstObject;
+		bool first = true;
+
+		if (SSL_use_certificate(_SSL,
+		    certificate.of_certificate) != 1 ||
+		    SSL_use_PrivateKey(_SSL, certificate.of_privateKey) != 1)
+			@throw [OFTLSHandshakeFailedException
+			    exceptionWithStream: self
+					   host: host
+				      errorCode: initFailedErrorCode];
+
+		for (OFOpenSSLX509Certificate *iter in _certificateChain) {
+			if (first) {
+				first = false;
+				continue;
+			}
+
+			if (SSL_add1_chain_cert(_SSL, iter.of_certificate) != 1)
+				@throw [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: host
+					      errorCode: initFailedErrorCode];
+		}
+	}
+
+	ERR_clear_error();
 	status = SSL_do_handshake(_SSL);
 
 	while (BIO_ctrl_pending(_writeBIO) > 0) {
@@ -298,20 +384,34 @@ static SSL_CTX *clientContext;
 	else {
 		switch (SSL_get_error(_SSL, status)) {
 		case SSL_ERROR_WANT_READ:
-			[_underlyingStream asyncReadIntoBuffer: _buffer
-							length: bufferSize
-						   runLoopMode: runLoopMode];
-			[_delegate retain];
-			objc_autoreleasePoolPop(pool);
-			return;
+			if (!_underlyingStream.atEndOfStream) {
+				[_underlyingStream
+				    asyncReadIntoBuffer: _buffer
+						 length: bufferSize
+					    runLoopMode: runLoopMode];
+				[_delegate retain];
+				objc_autoreleasePoolPop(pool);
+				return;
+			}
+
+			exception = [OFTLSHandshakeFailedException
+			    exceptionWithStream: self
+					   host: host
+				      errorCode: OFTLSStreamErrorCodeUnknown];
+			break;
 		case SSL_ERROR_WANT_WRITE:
 			[_underlyingStream asyncWriteData: [OFData data]
 					      runLoopMode: runLoopMode];
 			[_delegate retain];
 			objc_autoreleasePoolPop(pool);
 			return;
+		case SSL_ERROR_SSL:
+			exception = [OFTLSHandshakeFailedException
+			    exceptionWithStream: self
+					   host: host
+				      errorCode: errToErrorCode(_SSL)];
+			break;
 		default:
-			/* FIXME: Map to better errors */
 			exception = [OFTLSHandshakeFailedException
 			    exceptionWithStream: self
 					   host: host
@@ -320,15 +420,37 @@ static SSL_CTX *clientContext;
 		}
 	}
 
-	if ([_delegate respondsToSelector:
-	    @selector(stream:didPerformClientHandshakeWithHost:exception:)])
-		[_delegate		       stream: self
-		    didPerformClientHandshakeWithHost: host
-					    exception: exception];
+	if (server) {
+		if ([_delegate respondsToSelector: @selector(
+		    streamDidPerformServerHandshake:exception:)])
+			[_delegate streamDidPerformServerHandshake: self
+							 exception: exception];
+	} else {
+		if ([_delegate respondsToSelector: @selector(
+		    stream:didPerformClientHandshakeWithHost:exception:)])
+			[_delegate		       stream: self
+			    didPerformClientHandshakeWithHost: host
+						    exception: exception];
+	}
 
 	objc_autoreleasePoolPop(pool);
 }
 
+- (void)asyncPerformClientHandshakeWithHost: (OFString *)host
+				runLoopMode: (OFRunLoopMode)runLoopMode
+{
+	[self of_asyncPerformHandshakeWithHost: host
+					server: false
+				   runLoopMode: runLoopMode];
+}
+
+- (void)asyncPerformServerHandshakeWithRunLoopMode: (OFRunLoopMode)runLoopMode
+{
+	[self of_asyncPerformHandshakeWithHost: nil
+					server: true
+				   runLoopMode: runLoopMode];
+}
+
 -      (bool)stream: (OFStream *)stream
   didReadIntoBuffer: (void *)buffer
 	     length: (size_t)length
@@ -343,6 +465,7 @@ static SSL_CTX *clientContext;
 		OFEnsure(BIO_write(_readBIO, buffer, (int)length) ==
 		    (int)length);
 
+		ERR_clear_error();
 		status = SSL_do_handshake(_SSL);
 
 		while (BIO_ctrl_pending(_writeBIO) > 0) {
@@ -359,13 +482,26 @@ static SSL_CTX *clientContext;
 		else {
 			switch (SSL_get_error(_SSL, status)) {
 			case SSL_ERROR_WANT_READ:
-				return true;
+				if (!_underlyingStream.atEndOfStream)
+					return true;
+
+				exception = [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: _host
+					      errorCode: unknownErrorCode];
+				break;
 			case SSL_ERROR_WANT_WRITE:;
 				OFRunLoopMode runLoopMode =
 				    [OFRunLoop currentRunLoop].currentMode;
 				[_underlyingStream asyncWriteData: [OFData data]
 						      runLoopMode: runLoopMode];
 				return false;
+			case SSL_ERROR_SSL:
+				exception = [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: _host
+					      errorCode: errToErrorCode(_SSL)];
+				break;
 			default:
 				exception = [OFTLSHandshakeFailedException
 				    exceptionWithStream: self
@@ -376,11 +512,18 @@ static SSL_CTX *clientContext;
 		}
 	}
 
-	if ([_delegate respondsToSelector:
-	    @selector(stream:didPerformClientHandshakeWithHost:exception:)])
-		[_delegate		       stream: self
-		    didPerformClientHandshakeWithHost: _host
-					    exception: exception];
+	if (_server) {
+		if ([_delegate respondsToSelector: @selector(
+		    streamDidPerformServerHandshake:exception:)])
+			[_delegate streamDidPerformServerHandshake: self
+							 exception: exception];
+	} else {
+		if ([_delegate respondsToSelector: @selector(
+		    stream:didPerformClientHandshakeWithHost:exception:)])
+			[_delegate		       stream: self
+			    didPerformClientHandshakeWithHost: _host
+						    exception: exception];
+	}
 
 	[_delegate release];
 
@@ -407,6 +550,7 @@ static SSL_CTX *clientContext;
 			[_underlyingStream flushWriteBuffer];
 		}
 
+		ERR_clear_error();
 		status = SSL_do_handshake(_SSL);
 
 		while (BIO_ctrl_pending(_writeBIO) > 0) {
@@ -432,6 +576,12 @@ static SSL_CTX *clientContext;
 				return nil;
 			case SSL_ERROR_WANT_WRITE:
 				return data;
+			case SSL_ERROR_SSL:
+				exception = [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: _host
+					      errorCode: errToErrorCode(_SSL)];
+				break;
 			default:
 				exception = [OFTLSHandshakeFailedException
 				    exceptionWithStream: self
@@ -442,11 +592,18 @@ static SSL_CTX *clientContext;
 		}
 	}
 
-	if ([_delegate respondsToSelector:
-	    @selector(stream:didPerformClientHandshakeWithHost:exception:)])
-		[_delegate		       stream: self
-		    didPerformClientHandshakeWithHost: _host
-					    exception: exception];
+	if (_server) {
+		if ([_delegate respondsToSelector: @selector(
+		    streamDidPerformServerHandshake:exception:)])
+			[_delegate streamDidPerformServerHandshake: self
+							 exception: exception];
+	} else {
+		if ([_delegate respondsToSelector: @selector(
+		    stream:didPerformClientHandshakeWithHost:exception:)])
+			[_delegate		       stream: self
+			    didPerformClientHandshakeWithHost: _host
+						    exception: exception];
+	}
 
 	[_delegate release];
 
