@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -98,8 +98,16 @@
 	attributes = [OFDictionary
 	    dictionaryWithObject: date
 			  forKey: OFFileModificationDate];
-	[_fileManager setAttributes: attributes
-		       ofItemAtPath: _testFileIRI.fileSystemRepresentation];
+	@try {
+		[_fileManager
+		    setAttributes: attributes
+		     ofItemAtPath: _testFileIRI.fileSystemRepresentation];
+	} @catch (OFSetItemAttributesFailedException *e) {
+		if (e.errNo == ENOSYS)
+			OTSkip(@"Setting modification time not supported");
+
+		@throw e;
+	}
 
 	attributes = [_fileManager attributesOfItemAtPath:
 	    _testFileIRI.fileSystemRepresentation];
@@ -320,6 +328,11 @@
 		[_fileManager
 		    linkItemAtPath: sourceIRI.fileSystemRepresentation
 			    toPath: destinationIRI.fileSystemRepresentation];
+	} @catch (OFLinkItemFailedException *e) {
+		if (e.errNo == EPERM)
+			OTSkip(@"Links not supported");
+
+		@throw e;
 	} @catch (OFNotImplementedException *e) {
 		OTSkip(@"Links not supported");
 	}
