@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -596,36 +596,45 @@ static const char *range80ToFF =
 	    OFOutOfRangeException);
 }
 
-- (void)testIndexOfCharacterFromSet
+- (void)testRangeOfCharacterFromSet
 {
 	OFCharacterSet *characterSet =
 	    [OFCharacterSet characterSetWithCharactersInString: @"cđ"];
+	OFRange range;
 
-	OTAssertEqual([[self.stringClass stringWithString: @"abcđabcđe"]
-	    indexOfCharacterFromSet: characterSet], 2);
+	range = [[self.stringClass stringWithString: @"abcđabcđe"]
+	    rangeOfCharacterFromSet: characterSet];
+	OTAssertEqual(range.location, 2);
+	OTAssertEqual(range.length, 1);
 
-	OTAssertEqual([[self.stringClass stringWithString: @"abcđabcđë"]
-	    indexOfCharacterFromSet: characterSet
-			    options: OFStringSearchBackwards], 7);
+	range = [[self.stringClass stringWithString: @"abcđabcđë"]
+	    rangeOfCharacterFromSet: characterSet
+			    options: OFStringSearchBackwards];
+	OTAssertEqual(range.location, 7);
+	OTAssertEqual(range.length, 1);
 
-	OTAssertEqual([[self.stringClass stringWithString: @"abcđabcđë"]
-	    indexOfCharacterFromSet: characterSet
+	range = [[self.stringClass stringWithString: @"abcđabcđë"]
+	    rangeOfCharacterFromSet: characterSet
 			    options: 0
-			      range: OFMakeRange(4, 4)], 6);
+			      range: OFMakeRange(4, 4)];
+	OTAssertEqual(range.location, 6);
+	OTAssertEqual(range.length, 1);
 
-	OTAssertEqual([[self.stringClass stringWithString: @"abcđabcđëf"]
-	    indexOfCharacterFromSet: characterSet
+	range = [[self.stringClass stringWithString: @"abcđabcđëf"]
+	    rangeOfCharacterFromSet: characterSet
 			    options: 0
-			      range: OFMakeRange(8, 2)], OFNotFound);
+			      range: OFMakeRange(8, 2)];
+	OTAssertEqual(range.location, OFNotFound);
+	OTAssertEqual(range.length, 0);
 }
 
-- (void)testIndexOfCharacterFromSetFailsWithOutOfRangeRange
+- (void)testRangeOfCharacterFromSetFailsWithOutOfRangeRange
 {
 	OFCharacterSet *characterSet =
 	    [OFCharacterSet characterSetWithCharactersInString: @"cđ"];
 
 	OTAssertThrowsSpecific([[self.stringClass stringWithString: @"𝄞öö"]
-	    indexOfCharacterFromSet: characterSet
+	    rangeOfCharacterFromSet: characterSet
 			    options: 0
 			      range: OFMakeRange(3, 1)],
 	    OFOutOfRangeException);
@@ -749,6 +758,10 @@ static const char *range80ToFF =
 
 	OTAssertEqual([[self.stringClass stringWithString:
 	    @"\t\t\r\n"] longLongValueWithBase: 8], 0);
+
+	OTAssertEqual([[self.stringClass stringWithString:
+	    ([OFString stringWithFormat: @"%lld", LLONG_MIN])] longLongValue],
+	    LLONG_MIN);
 }
 
 - (void)testLongLongValueThrowsOnInvalidFormat
@@ -1389,8 +1402,8 @@ static const char *range80ToFF =
 
 	OTAssertEqualObjects([[self.stringClass stringWithString: @""]
 	    pathComponents], [OFArray array]);
-# elif defined(OF_NINTENDO_3DS) || defined(OF_WII) || \
-    defined(OF_NINTENDO_SWITCH)
+# elif defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS) || \
+    defined(OF_WII) || defined(OF_NINTENDO_SWITCH)
 	OTAssertEqualObjects([[self.stringClass stringWithString:
 	    @"sdmc:/tmp"] pathComponents],
 	    ([OFArray arrayWithObjects: @"sdmc:", @"tmp", nil]));
@@ -1565,8 +1578,8 @@ static const char *range80ToFF =
 	    @"foo/bar"] stringByDeletingLastPathComponent], @"foo");
 	OTAssertEqualObjects([[self.stringClass stringWithString:
 	    @"foo"] stringByDeletingLastPathComponent], @"");
-# elif defined(OF_NINTENDO_3DS) || defined(OF_WII) || \
-    defined(OF_NINTENDO_SWITCH)
+# elif defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS) || \
+    defined(OF_WII) || defined(OF_NINTENDO_SWITCH)
 	OTAssertEqualObjects([[self.stringClass stringWithString:
 	    @"/tmp/"] stringByDeletingLastPathComponent], @"");
 	OTAssertEqualObjects([[self.stringClass stringWithString:
