@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -22,6 +22,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include "unistd_wrapper.h"
+
 #import "ObjFW.h"
 #import "ObjFWTest.h"
 
@@ -68,6 +70,11 @@
 	}
 
 	@try {
+#ifndef OF_WINDOWS
+		OFUNIXSocketCredentials peerCredentials;
+		OFNumber *number;
+#endif
+
 		[sockServer listen];
 
 		[sockClient connectToPath: path];
@@ -80,6 +87,26 @@
 
 		OTAssertEqual(OFSocketAddressUNIXPath(
 		    sockAccepted.remoteAddress).length, 0);
+
+#ifndef OF_WINDOWS
+		peerCredentials = sockAccepted.peerCredentials;
+
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsUserID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getuid()]);
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsGroupID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getgid()]);
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsProcessID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getpid()]);
+#endif
 	} @finally {
 #ifdef OF_HAVE_FILES
 		[[OFFileManager defaultManager] removeItemAtPath: path];
