@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -32,9 +32,6 @@
 #import "OFIRI.h"
 #import "OFLocale.h"
 #import "OFOptionsParser.h"
-#ifdef OF_HAVE_PLUGINS
-# import "OFPlugin.h"
-#endif
 #import "OFSandbox.h"
 #import "OFStdIOStream.h"
 #import "OFSystemInfo.h"
@@ -396,17 +393,18 @@ fileNameFromContentDisposition(OFString *contentDisposition)
 		    rangeOfString: @":"
 			  options: OFStringSearchBackwards].location;
 		OFString *host;
-		unsigned long long port;
+		unsigned short port;
 
 		if (pos == OFNotFound)
 			@throw [OFInvalidFormatException exception];
 
 		host = [proxy substringToIndex: pos];
-		port = [proxy substringFromIndex: pos + 1]
-		    .unsignedLongLongValue;
+		port = [proxy substringFromIndex: pos + 1].unsignedShortValue;
 
-		if (port > UINT16_MAX)
+#if USHRT_MAX != 65535
+		if (port > 65535)
 			@throw [OFOutOfRangeException exception];
+#endif
 
 		[OFTCPSocket setSOCKS5Host: host];
 		[OFTCPSocket setSOCKS5Port: (uint16_t)port];
